package com.ruoyi.demo.utils;

import cn.hutool.core.lang.Console;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.utils.redis.RedisUtils;
import com.ruoyi.demo.constant.RedisContant;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.mapper.WdInfoDao;
import org.apache.ibatis.cursor.Cursor;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.redisson.api.GeoUnit;
import org.redisson.api.RFuture;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.stream.Collectors;

@Component
public class WdRedisStoreage {
    @Autowired
    private  RedisTemplate redisTemplate;

    @Autowired
    public WdInfoDao wdInfoDao;

    @Autowired
    private SqlSessionFactory sqlSessionFactory;

    @Autowired
    @Qualifier("executor")
    private ExecutorService executorService;




    //@Autowired
    //WriteService writeService;


    /***
     * 获取中心网点的周边标签
     * @param wdInfo 中心网点信息
     * @return
     */
    public  List<String> getWdTag(WdInfo wdInfo){
        List<String> tag = RedisUtils.getCacheMapValue(RedisContant.WD_TAG_MAP, RedisContant.WD_TAG + "_" + wdInfo.getWdId());
        //List<String> tag = (List<String>) redisTemplate.boundValueOps(RedisContant.WD_TAG+"_"+wdInfo.getWdId()).get();
        return tag;
    }

    public  List<List<String>> getWdTagList(List<String> key){
        Map<String, List<String>> tags = RedisUtils.getMultiCacheMapValue(RedisContant.WD_TAG_MAP, new HashSet<>(key));
        return new ArrayList<>(tags.values());
    }

    /***
     * 分析网点周边标签，并把分析结果存入redis（默认范围：1km）
     * @param wdInfo 中心网点信息
     */
    public void wdTagAnalyse(WdInfo wdInfo) throws ExecutionException, InterruptedException {
        //1. 找到距离中心网点半径1km的所有网点Id
        RFuture<Map<Object, Double>> radius = RedisUtils.getClient().getGeo(wdInfo.getAddrCode().substring(0, 4)).radiusWithDistanceAsync(wdInfo.getWdId(), 1, GeoUnit.KILOMETERS);
        CompletableFuture<List<String>> completableFuture = CompletableFuture.supplyAsync(() -> {
            try {
                Map<Object, Double> distanceMap = radius.get();
                List<String> aroundWdId = distanceMap.keySet().stream().map(item -> (String) item).collect(Collectors.toList());
                //2. 查询周边网点标签的Set集
                QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.select("type_name_by")
                    .in("wd_id", aroundWdId)
                    .groupBy("type_name_by");
                List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);
                //3. 打标签
                List<String> tag = new ArrayList<>();
                wdInfos.forEach(item -> {
                    if (item != null) {
                        String typeNameBy = item.getTypeNameBy();
                        if (StrUtil.isNotEmpty(typeNameBy)) {
                            String[] split = typeNameBy.split(":");
                            if (split.length > 0)
                                tag.add(split[split.length - 1]);
                        }
                    }
                });

                RedisUtils.setCacheMapValue(RedisContant.WD_TAG_MAP,RedisContant.WD_TAG + "_" + wdInfo.getWdId(), tag);
                return tag;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }, executorService);

        completableFuture.get();

    }

    /**
     * 将网点数据库 导入 redis
     * */
    public void writeWdGeoRedis(){
        //清重
        //clearSql2();

        //将所有网点坐标导入redis
        //writeWdGeoRedis1("2023-09-25");

        //给网点打标签并存入redis中
        //writeWdGeoRedis2();
    }
//
//    public void clearSql2(){
//        List<String> wdId = new ArrayList<>();
//        List<String> storeId = new ArrayList<>();
//        List<String> houseId = new ArrayList<>();
//        List<String> buildId = new ArrayList<>();
//        List<String> trafficId = new ArrayList<>();
//        List<String> enterpriseId = new ArrayList<>();
//        long start = System.currentTimeMillis();
//
//        try{
//            System.out.println("开始清理");
//
//            QueryWrapper<WdInfo> wdInfoOriginQueryWrapper = new QueryWrapper<>();
//            wdInfoOriginQueryWrapper.select("hash", "count(hash) as audit").groupBy("hash").having("count(hash) >= 2");
//            List<WdInfo> wdInfoOrigins = wdInfoDao.selectList(wdInfoOriginQueryWrapper);
//
//            HashMap<String, Integer> hashMap = new HashMap<>();
//            for (WdInfo wdInfo : wdInfoOrigins) {
//                hashMap.put(wdInfo.getHash(), wdInfo.getAudit());
//            }
//
//
//            HashMap<String,Integer> hashMap1 = new HashMap<>();
//            hashMap1.put("total",1);
//            CompletableFuture<Void> future = null;
//            for (String s : hashMap.keySet()) {
//                FutureTask
//                future = CompletableFuture.runAsync(()->{
//                    synchronized (this){
//                        if(hashMap1.get("total")%1000 == 0){
//                            writeService.clearSql2(wdId,storeId,houseId,buildId,trafficId,enterpriseId);
//                            wdId.clear();
//                            storeId.clear();
//                            houseId.clear();
//                            buildId.clear();
//                            trafficId.clear();
//                            enterpriseId.clear();
//                        }
//                    }
//
//                    QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
//                    queryWrapper.eq("hash",s);
//                    queryWrapper.orderByDesc("collect_time");
//                    queryWrapper.last("limit " + (hashMap.get(s) - 1));
//                    List<WdInfo> wdInfos = wdInfoDao.selectList(queryWrapper);
//                    for (WdInfo wdInfo : wdInfos) {
//                        wdId.add(wdInfo.getWdId());
//
//                        switch (wdInfo.getWdTypeCode()){
//                            case "1":
//                                storeId.add(wdInfo.getWdId());
//                                break;
//                            case "2":
//                                houseId.add(wdInfo.getWdId());
//                                break;
//                            case "3":
//                                buildId.add(wdInfo.getWdId());
//                                break;
//                            case "4":
//                                trafficId.add(wdInfo.getWdId());
//                                break;
//                            case "5":
//                                enterpriseId.add(wdInfo.getWdId());
//                                break;
//                            default:
//                                break;
//                        }
//                    }
//                    System.out.println("进度："+wdInfoOrigins.size()+"/"+hashMap1.get("total"));
//                    hashMap1.put("total",hashMap1.get("total")+1);
//                },executorService);
//            }
//            future.get();
//        }catch (Exception e){
//            e.printStackTrace();
//        }finally {
//            //删除
//            writeService.clearSql2(wdId,storeId,houseId,buildId,trafficId,enterpriseId);
//            long end = System.currentTimeMillis();
//            System.out.println("完成清理 "+(end-start)/(1000*60));
//        }
//
//
//    }

    public void writeWdGeoRedis1(String time) throws Exception{
        //1.将网点坐标存入redis中
        long start = System.currentTimeMillis();
        System.out.println("开始将网点坐标存入redis");
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.ge("collect_time",time);
        Long total = wdInfoDao.selectCount(queryWrapper);

        queryWrapper.select("wd_id","addr_code","lng","lat");

        //使用sqlSessionFactory打开一个sqlSession，在没有读取完数据之前不要提交事务或关闭sqlSession
        System.out.println("----开启sqlSession");
        SqlSession sqlSession = sqlSessionFactory.openSession();
        Cursor<WdInfo> cursor = null;
        try {
            //获取到指定mapper
            WdInfoDao mapper = sqlSession.getMapper(WdInfoDao.class);
            //调用指定mapper的方法，返回一个cursor
            cursor = mapper.streamQuery(queryWrapper);

            if (cursor != null) {
                for (WdInfo wdInfo : cursor) {
                    //打印进度条
                    Console.printProgress('=', 50, (double) (cursor.getCurrentIndex() + 1) / total);

                    RedisUtils.getClient().getGeo(wdInfo.getAddrCode().substring(0, 4)).add(wdInfo.getLng().doubleValue(), wdInfo.getLat().doubleValue(), wdInfo.getWdId());
                    wdTagAnalyse(wdInfo);
                }
                if (cursor.isConsumed()) {
                    System.out.println("----查询sql匹配中的数据已经消费完毕！");
                }
            }
            sqlSession.commit();
            System.out.println("----提交事务");
        } catch (Exception e) {
            e.printStackTrace();
            sqlSession.rollback();
        } finally {
            if (cursor != null && cursor.isOpen()) {
                //关闭cursor
                cursor.close();
                System.out.println("----关闭cursor");
            }
            if (sqlSession != null) {
                //全部数据读取并且做好其他业务操作之后，提交事务并关闭连接；
                sqlSession.close();
                System.out.println("----关闭sqlSession");
            }

        }

        long end = System.currentTimeMillis();
        System.out.println("完成存入 耗时："+(end-start)/(1000*60) + "分钟");
    }

    public void writeWdGeoRedis2(String[] wdTypeCodes) throws IOException {
        //对所有网点坐标进行分析打标签
        long start = System.currentTimeMillis();
        System.out.println("开始打标签");

        //开始
        QueryWrapper<WdInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("wd_type_code", wdTypeCodes);
        Long total = wdInfoDao.selectCount(queryWrapper);

        queryWrapper.select("wd_id","addr_code");


        //使用sqlSessionFactory打开一个sqlSession，在没有读取完数据之前不要提交事务或关闭sqlSession
        System.out.println("----开启sqlSession");
        SqlSession sqlSession = sqlSessionFactory.openSession();
        Cursor<WdInfo> cursor = null;
        try {
            //获取到指定mapper
            WdInfoDao mapper = sqlSession.getMapper(WdInfoDao.class);
            //调用指定mapper的方法，返回一个cursor
            cursor = mapper.streamQuery(queryWrapper);

            if (cursor != null) {
                for (WdInfo wdInfo : cursor) {
                    //打印进度条
                    Console.printProgress('=', 50, (double) (cursor.getCurrentIndex() + 1) / total);
                    wdTagAnalyse(wdInfo);
                }
                if (cursor.isConsumed()) {
                    System.out.println("----查询sql匹配中的数据已经消费完毕！");
                }
            }
            sqlSession.commit();
            System.out.println("----提交事务");
        } catch (Exception e) {
            e.printStackTrace();
            sqlSession.rollback();
        } finally {
            if (cursor != null && cursor.isOpen()) {
                //关闭cursor
                cursor.close();
                System.out.println("----关闭cursor");
            }
            if (sqlSession != null) {
                //全部数据读取并且做好其他业务操作之后，提交事务并关闭连接；
                sqlSession.close();
                System.out.println("----关闭sqlSession");
            }

        }

        long end = System.currentTimeMillis();
        System.out.println("完成打标签 耗时："+(end-start)/(1000*60) + "分钟");

    }


}
